import types
from functools import update_wrapper, partial
from datetime import date, time
import locale

import agenda2pdf
import autocomplete_light
from django.conf import settings
from django.contrib import admin
from django.contrib.admin import widgets
from django import forms
from django.db import models as dbmodels
from django.utils.translation import ugettext as _
from django.contrib import messages
from django.utils.timezone import now

from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponseForbidden
from autocomplete_light.widgets import ChoiceWidget, MultipleChoiceWidget
from django_filters import filters as dj_filters

from jmb.ticket import models 
from jmb.ticket.admin import (
    Ticket, TicketAdmin, TicketProject, PlanAdmin, 
    TicketProjectAdmin, TicketProjectAdminForm, Plan
)
from cogema.contratto.models import CogemaProject, Contratto

locale.setlocale(locale.LC_TIME, 'it_IT.UTF-8')
admin.site.unregister(Ticket)
admin.site.unregister(TicketProject)


## Label personalizzata per ticket
def ticket_label(self):
    addr = self.get_address()
    if not addr:
        return ''
    return "%s %s" % (addr.name, addr.locality) 

def ticket_description(self):
    return u'<em>%s</em><br>%s' % (self.title, self.description)

def description_strip(self):
    return self.description.replace('\n',' ').replace('\t',' ')
    
def ticket_plans_description_pdf(self):
    txt = []
    for plan in self.plans.all().order_by('date_begin'):
        txt += [u'\u25c9 %s' % plan.description]
    return u''.join(txt)
    
def ticket_plans_description(self):
    txt = []
    for plan in self.plans.all().order_by('date_begin'):
        txt += [u'<br> \u25c9 %s %s <br>' % (plan.date_begin.strftime('%d/%m/%y'), plan.description)]
    return u''.join(txt) 

Ticket.label = types.MethodType(ticket_label, None, Ticket)
Ticket.label_description = types.MethodType(ticket_description, None, Ticket)
Ticket.ticket_plans_description = types.MethodType(ticket_plans_description, None, Ticket)
Ticket.ticket_plans_description_pdf = types.MethodType(ticket_plans_description_pdf, None, Ticket)
Ticket.description_strip = types.MethodType(description_strip, None, Ticket)

# class CogemaContact(models.TicketContact):
#     class Meta:
#         proxy = True

#     @property
#     def ticket_organizations(self):
#         return models.Organization.customers_owners.filter()

#     def get_organization_ticketprojects(self, organization):
#         return TicketProject.active.filter(
#             organization_id=organization.id
#         ).distinct()

#models.Contact = CogemaContact    

class CogemaOrganization(models.Organization):
    class Meta:
        proxy = True

    @property
    def ticket_submitters(self):
        return Contact.objects.filter(
            status=1
        ).distinct()

models.Organization = CogemaOrganization

## Label personalizzata per plan
def label_plan(self):
    workers = "<br>".join([str(w) for w in self.workers.all()])
    luogo =  self.get_address() #self.ticket.label()
    chi_inserito_ticket = "%s - %s" % (
        self.ticket.creator, self.ticket.date_create.strftime("%d/%m/%y %H:%M"))
    if not self.ticket.last_modifier_id == self.ticket.creator_id:
        modificato_da = "Modificato da: %s" % self.ticket.last_modifier
    else:
        modificato_da = ''
    description = self.ticket.description
    plans_description = self.ticket.ticket_plans_description()
    ticket_id = self.ticket.id
    plan_id = self.id
    title = self.ticket.title
    txt = u"""
    <b>%(workers)s</b><br>
    <div class="hideprint" ><a class="iframe" href="/admin/ticket/ticket/%(ticket_id)s/detail/?_popup=1" target="_blank"><i class="fa fa-search-plus"></i></a>
    <a href="mailto:%(ticket_id)s@ticket.local" ><i class="fa fa-paperclip"></i></a>
    <a href="/admin/ticket/plan/pdf/%(plan_id)s/?_popup=1" ><i class="fa fa-print"></i></a>
    <br><em>%(chi_inserito_ticket)s</em></div>
    %(luogo)s
    <br><b>%(title)s</b>
    <br>%(description)s
    <br>%(plans_description)s
    <br>%(modificato_da)s""" % locals()
    return txt


Plan.label = types.MethodType(label_plan, None, Plan)

class Event(object):
    def __init__(self, plan):
        
        self.year = plan.date_begin.year
        self.month = plan.date_begin.month
        self.day = plan.date_begin.day
        self.time =  time(plan.date_begin.hour,plan.date_begin.minute)
        addr = plan.get_address()
        self.address = u'%s %s' % (addr.name, addr.locality) 
        self.text = plan.description + " " + plan.ticket.description
        self.title = plan.ticket.title

def create_agenda(model_admin, request, queryset):
    appointments = list()
    dw = 1
    year = queryset[0].date_begin.year
    month = queryset[0].date_begin.month
    day = queryset[0].date_begin.day
    for obj in queryset.order_by('date_begin'):
        if obj.date_begin.strftime("%y%m%d") != queryset[0].date_begin.strftime("%y%m%d"):
            messages.error(request, "non puoi mettere eventi di giorni diversi") # tradurre
            return False
        evt = Event(obj)
        appointments.append(evt)
    cfg = agenda2pdf.Config()
    cfg.year = year
    cfg.output = settings.MEDIA_ROOT + '/agenda.pdf'
    cfg.compression = 0
    cfg.first_week_day = 0
    cfg.format = 'd'
    canvas = agenda2pdf.prepare_canvas(cfg)
    cal = agenda2pdf.calendar.Calendar(cfg.first_week_day)
    agenda2pdf.draw_day(canvas, year, month, day, dw, appointments, cfg, cal)
    canvas.save()
    return HttpResponseRedirect(settings.MEDIA_URL + '/agenda.pdf')

create_agenda.short_description = _('Generate Agenda pdf')

class RangeWithTomorrow(dj_filters.DateRangeFilter):
    options = dj_filters.DateRangeFilter.options.copy()
    options[0] = (_('Tomorrow'), lambda qs, name: qs.filter(**{
            '%s__year' % name: now().year,
            '%s__month' % name: now().month,
            '%s__day' % name: now().day +1,
        }))
    

class CogemaPlanAdmin(PlanAdmin):
    actions = [create_agenda]
    def get_filterset_class(self, request):
        PlanFilterset = super(CogemaPlanAdmin, self).get_filterset_class(request)
        class CogemaPlanFilters(PlanFilterset):
            date_begin__range = RangeWithTomorrow(name='date_begin', lookup_type='range', label='Inizio')
        return CogemaPlanFilters
    
class CogemaTicketAdmin(TicketAdmin):

    advanced_search_fields = (
        ('id', 'priority'),
        ('title__icontains', 'description__icontains',),
        ('project__name__icontains:progetto', 'organization__name__icontains:organization',),
        (
            'project__organization__addresses__zip_code',
            'project__organization__addresses__name:indirizzo',
        ),
        ('status__in', 'category__in',),
        ('keywords', 'typology__in',),
        ('date_create__gte', 'date_create__lte',),
        ('triggerline__gte', 'triggerline__lte',),
        ('deadline__gte', 'deadline__lte', 
         'project__organization__azienda__amministratore:amministratore'),
    )
    def get_filterset_class(self, request):
        TkFilterset = super(CogemaTicketAdmin, self).get_filterset_class(request)
        class CogemaTkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(CogemaTkFilters, self).__init__(*args, **kw)
                widget_with_autocompl = autocomplete_light.ChoiceWidget('amministratore')
                self.form.fields['project__organization__azienda__amministratore'].widget = widget_with_autocompl

        return CogemaTkFilters

    def get_advanced_search_fields(self, *args, **kwargs):
        return self.advanced_search_fields
    
    
    def get_form(self, request, obj=None, **kwargs):
        class_ = super(CogemaTicketAdmin, self).get_form(request, obj=None, **kwargs)
        class CogemaTicketAdminForm(class_):
            description = dbmodels.TextField()
            def __init__(self, *args, **kwargs):
                class_.__init__(self, *args, **kwargs)
                self.fields['description'].widget = forms.Textarea(attrs={'class': 'vLargeTextField'})
        return CogemaTicketAdminForm
        #class_.base_fields['description'].widget = widgets.AdminTextareaWidget
        #return class_

class CogemaTicketProjectAdminForm(TicketProjectAdminForm):
    def __init__(self, *args, **kwargs):
        super(CogemaTicketProjectAdminForm, self).__init__(*args, **kwargs)
        self.fields['managers'].required = False
        self.fields['staff'].required = False
        self.fields['organization'].label = _("Organization").capitalize()
        self.fields['address'].label = _("Address")
        organization_id = self.get_object_id('organization')
        contratto_qs = Contratto.objects.none()

        if organization_id:
            contratto_qs = Contratto.objects.filter(cliente__id=organization_id)

        self.fields['contratto'] = forms.ModelChoiceField(
            queryset=contratto_qs, required=False,
            widget=ChoiceWidget('Contratto')
        )

    # def clean_managers(self):
    #     if not self.cleaned_data['managers']:
    #         self.cleaned_data['managers'] = models.Contact.owners.filter(
    #             role__name__in=['Tecnico', 'Amministrativo', 'Direzione', 'Fatturazione'])
    #     return self.cleaned_data['managers']

    class Meta:
        model = CogemaProject


class CogemaTicketProjectAdmin(TicketProjectAdmin):
    form = CogemaTicketProjectAdminForm
    list_display = (
        'status', 'date_create',  'date_start', 'date_end',
        'organization', 'name', 'creator', 'description',
#        'get_string_managers', 'get_string_staff', 'get_string_customers',
#        'assistance_email',
        'get_edit_icon', 'get_delete_icon'
    )
    
    fieldsets = (
        (_('general admin'), {
            'fields': (
                ('status', 'name'),
                ('organization', 'address', ),
                ('description', 'contratto'),
                ('managers', 'assistance_email'), 
                ('staff', 'customers'),
                ('date_start', 'date_end'), 
            )
        }),
    )

    advanced_search_fields = (
        ('name__icontains:project', 'organization__name__icontains:organization'),
        ('description__icontains',),
        ('date_create__gte', 'date_create__lte',),
        ('date_start__gte', 'date_start__lte',),
#        ('managers', 'staff')
    )
    
    tabs = (
        (_('Project'), {'items': [_('general admin')], 'active': True}),
    )
    
    def get_readonly_fields(self, request, obj=None):
        return ('contratto',) + super(CogemaTicketProjectAdmin, self).get_readonly_fields(request, obj=None)
        
def get_ticket_staff(q=None, json=False):
    """
    Return a set of all contacts in staff of all tickets in q

    :arg q: a QuerySet of tickets as returned by an advanced_search 
            of ticket changelist
    :arg json: boolean. If True return a json serialized result. Default False.
    """
    from django.core import serializers
    from jmb.ticket.models import Contact

    contacts = list(Contact.owners.all().order_by('role__ordering', 'last_name'))
    esterni = list(Contact.objects.filter(role__name='Operativo_EXT').order_by('role__ordering', 'last_name'))
    contacts += esterni

    # Probabilmente non particolarmente efficente, remotamente usa IN
    # select_related non funziona con many2many
    if q:
        q = q.filter(project__isnull=False).prefetch_related('project__staff')

        for ticket in q.all():
            for contact in ticket.project.staff.all():
                if not contact in contacts:
                    contacts += [contact]
    if json:
        return serializers.serialize('json', contacts, extras=('get_full_name',))
    else:
        return contacts

models.get_ticket_staff = get_ticket_staff


admin.site.register(CogemaProject, CogemaTicketProjectAdmin)
admin.site.register(Ticket, CogemaTicketAdmin)
admin.site.unregister(Plan)
admin.site.register(Plan, CogemaPlanAdmin)

